#!/usr/bin/perl

# acl.pl - Advanced Color Logs
# version 0.4.0
# written by Patrick Mullen (p_mullen@linuxmail.org)
# adapted from ColorLogs by J-Dog (J-Dog@Resentment.org)
#
# VISIT THE LINUX RESOURCE CENTER -- http://www.LinuxRC.org
#
# The LATEST VERSION of this program can be found at
# http://www.LinuxRC.org/projects/acl 
#
# USAGE
# acl.pl [-f <configfile>] < file.to.colorize
#
# Default config file is /etc/acl.conf, alternate config file
# may be specified using the -f command-line parameter.

use Getopt::Long;

# Create the colors Assoc. Array
%colors = (
    'default'            => "\033[0m",
    'black'              => "\033[30m",
    'red'                => "\033[31m",
    'green'              => "\033[32m",
    'yellow'             => "\033[33m",
    'blue'               => "\033[34m",
    'magenta'            => "\033[35m",
    'cyan'               => "\033[36m",
    'white'              => "\033[37m"
);

%attributes = (
    'normal'             => "\033[00m",
    'bright'             => "\033[01m",
    'underlined'         => "\033[04m",
    'blinking'           => "\033[05m",
    'background'         => "\033[07m",
    'beep'               => "\007"
);   

$normal = $colors{default};

GetOptions("f:s" => \$configfile);

if($configfile eq "") { $configfile = "/etc/acl.conf"; }

open CFG, $configfile or die "Error opening configuration file $configfile.  Exiting.\n";

$maxindex=0;

while(<CFG>) {
   chomp;
        # Chomp out the whitespace
        s/^\s*//;
        s/\s*$//;

        # Search for special characters
        s/\~/\\\~/g;
#        s/\!/\\\!/g;   # We use ! to denote a line is NOT to have a string
			# May want to make this smarter, ie don't substitute
			# if we find ",!" or ",[whitespace]!"
        s/\@/\\\@/g;
        s/\#/\\\#/g;
        s/\$/\\\$/g;
        s/\%/\\\%/g;
        s/\^/\\\^/g;
        s/\&/\\\&/g;
        s/\*/\\\*/g;
        s/\-/\\\-/g;
        s/\_/\\\_/g;
        s/\=/\\\=/g;
        s/\+/\\\+/g;
        s/\[/\\\[/g;
        s/\]/\\\]/g;
        s/\{/\\\{/g;
        s/\}/\\\}/g;
        s/\|/\\\|/g; 
        s/\"/\\\"/g;
        s/\;/\\\;/g;
        s/\</\\\</g;
        s/\>/\\\>/g;
        s/\?/\\\?/g;
        s/\(/\\\(/g;
        s/\)/\\\)/g;
        s/\`/\\\`/g;
        s/\'/\\\'/g;

        next if (/^:/);      

   ($colorconfig, $rest_of_line) = split(/\s*,(.*)/);
 
   $_ = $rest_of_line;
   ($primary_match, $secondary_matches) = split(/\s*,(.*)/);

   # Construct the output color by finding the color then attributes.

   if      ($colorconfig =~ /black/i)   { $color = $colors{black};
   } elsif ($colorconfig =~ /red/i)     { $color = $colors{red};
   } elsif ($colorconfig =~ /green/i)   { $color = $colors{green};
   } elsif ($colorconfig =~ /yellow/i)  { $color = $colors{yellow};
   } elsif ($colorconfig =~ /blue/i)    { $color = $colors{blue};
   } elsif ($colorconfig =~ /magenta/i) { $color = $colors{magenta};
   } elsif ($colorconfig =~ /cyan/i)    { $color = $colors{cyan};
   } elsif ($colorconfig =~ /white/i)   { $color = $colors{white}; 
   } else                               { $color = $colors{default};
   }

   if($colorconfig =~ /normal/i)     { $color = $color.$attributes{normal};     }
   if($colorconfig =~ /bright/i)     { $color = $color.$attributes{bright};     }
   if($colorconfig =~ /underlined/i) { $color = $color.$attributes{underlined}; }
   if($colorconfig =~ /blinking/i)   { $color = $color.$attributes{blinking};   }
   if($colorconfig =~ /background/i) { $color = $color.$attributes{background}; }
   if($colorconfig =~ /beep/i)       { $color = $color.$attributes{beep};       }

   $search_strings{$maxindex, color} = $color;
   $search_strings{$maxindex, primary_match} = $primary_match;
   $search_strings{$maxindex, secondary_matches} = $secondary_matches;

   $maxindex++;
}


# The main loop.  Process each input line and print them in color.

while($line = <>) {

   for($index = 0; $index < $maxindex && found == 0; $index++) {

      $color=$search_strings{$index, color};
      $primary_match = $search_strings{$index, primary_match};
      $secondary_matches=$search_strings{$index, secondary_matches};

      if($line =~ /$primary_match/) {

         $found=1;

         $rest_of_line=$secondary_matches;

         while($rest_of_line) {
            $_ = $rest_of_line;
            ($string, $rest_of_line) = split(/\s*,(.*)/);

            if(grep(/^!/, $string)) {
               if($line =~ substr($string, 1)) {
                  $found=0;
               } 
            }  else {
               if($line !~ $string) {
                  $found=0;
               }
            }
         }

         if($found == 1) {
            chomp $line;
            print "$color$line$normal\n";
            $found = 0;
            last;
         } 
      }
   }

   # If didn't find a match, just print the line
   if($found == 0 && $index == $maxindex) {
      print $line;
   }
}

